!----------------------------------MUDULE interpolate_C6_LUT_m--------------------------------------
!****FOR LUTS FLIPPED, with derived types*********
!Fortran 90 module that contains necessary routines to read and process Multi Scattering part of the
!Reflectance, To get the reflectance for a given solar, view  and rel azimuth angles and wind speed,
!Mulit Scattering part of the reflectance will be calculated via interpolation of data from the
!libraries and single scattering part will be then added by using the !analytical expression. 
!Here for aerosol layer Henyey-Greenstein phase function is used
!Data Files are stored in HDF. Three separate HDFs for Multi Scattered part of  Reflectance of
!water and Ice for wind speeds of 3,7,15, and a single HDF for the phase function data. 
!Main inputs will be the file names of the HDF files.
!If you need to look at the reflectances of water cloud for a given solar, view, azm and wind speed, 
!a usual order of calls would be (look at testLUTPGMS.f90)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!PROCEDURES INCLUDED:
!FUNCTIONS:
!	REAL FUNCTION ScatAngle						!calculate the scattering angle
!SUBROUTINES:
!   SUBROUTINE nullifyPointers                  !Nullify pointers, in derived types
!   SUBROUTINE pfdata_read                      !Read phase functions related data
!   SUBROUTINE refl_read                        !Read MS Reflectances, Fluxes
!	SUBROUTINE GetSDSArray						!overloaded routine to read 1D through 6D sds's
!	SUBROUTINE SetLogFileUnit					!set the file unit for log file (not used now)
!	SUBROUTINE WriteComments					!output comments to the log file (not used now)	  
!	SUBROUTINE getAllPhaseFunctionValues		!interpoltae phase funcion values for a given 
!												!scattering angle
!	                                                             
!	SUBROUTINE CalcSingScatPartOfOceanReflectanceNEW !calculate the single scattering part of the 
!											   	      reflectance
!	SUBROUTINE InterpolateFluxes				!interpolate Fluxes
!	SUBROUTINE GetReflForGivenSurfAlbedoNew		!calc reflectance for a given surface albedo
!PROGRAMMER:
!
!             Nandana Amarasinghe (SSAI)
!             Climate and Radiation Branch
!             NASA Goddard Space Flight Center
!             Greenbelt, Maryland 20771, U.S.A.
!             nandana.amarasinghe@ssaihq.com or nandana.d.amarasinghe@nasa.gov
!---------------------------------------------------------------------------------------------------

MODULE interpolate_C6_LUT_m
IMPLICIT NONE

REAL*8, PARAMETER						:: 	CONST_PI	=	3.14159265358979d0,& ! 4.d0 * ATAN(1.d0), &!value of PI 3.14159265358979d0,& !
						   					RAD_PER_DEG = CONST_PI/180.d0,  &	!radians per degree	
						   					DEG_PER_RAD = 180.d0/CONST_PI,&		!degrees per radian
						   					EXP_TOL = 12.0
REAL,PARAMETER,DIMENSION(3) 			:: WIND_SPEED = (/3.0,7.0,15.0/)						   					


INTEGER									::	lu_PGM_status   !file unit for the log file

												
                    
INCLUDE "hdf.f90"
INCLUDE "dffunc.f90"
     
PUBLIC GetSDSArray
INTERFACE GetSDSArray
	MODULE PROCEDURE Get_1D_SDS
	MODULE PROCEDURE Get_2D_SDS
	MODULE PROCEDURE Get_3D_SDS
	MODULE PROCEDURE Get_4D_SDS
	MODULE PROCEDURE Get_5D_SDS	
	MODULE PROCEDURE Get_6D_SDS
END INTERFACE




TYPE lutRefl
    REAL, DIMENSION(:,:,:,:,:,:), POINTER::msr
    REAL, DIMENSION(:),           POINTER::solzen
    REAL, DIMENSION(:),           POINTER::senzen
    REAL, DIMENSION(:),           POINTER::relazm
    INTEGER, DIMENSION(2)               ::idsol
    INTEGER, DIMENSION(2)               ::idzen
    INTEGER, DIMENSION(2)               ::idazm    
END TYPE lutRefl

TYPE lutFlux
    REAL, DIMENSION(:),           POINTER::fluxsol
    REAL, DIMENSION(:,:,:,:),     POINTER::rflux
    REAL, DIMENSION(:,:,:,:),     POINTER::tflux
    REAL, DIMENSION(:,:,:),       POINTER::spalb
    INTEGER, DIMENSION(2)               ::idsolf
    INTEGER, DIMENSION(2)               ::idzenf    
END TYPE lutFlux

TYPE lutAero
    REAL, DIMENSION(:),           POINTER::aerog
    REAL, DIMENSION(:),           POINTER::aerossa
    REAL, DIMENSION(:),           POINTER::aerocot
    REAL, DIMENSION(:),           POINTER::raycot
END TYPE lutAero

TYPE pfData
    REAL, DIMENSION(:,:,:),       POINTER::liqpf
    REAL, DIMENSION(:,:,:),       POINTER::icepf
    REAL, DIMENSION(:),           POINTER::liqsa
    REAL, DIMENSION(:),           POINTER::icesa
END TYPE pfData

TYPE lutSSP
    REAL, DIMENSION(:),           POINTER::wvl
    REAL, DIMENSION(:),           POINTER::re
    REAL, DIMENSION(:),           POINTER::cot
    REAL, DIMENSION(:,:),         POINTER::pfconst
    REAL, DIMENSION(:,:),         POINTER::truncf
    REAL, DIMENSION(:,:),         POINTER::ssalb
    REAL, DIMENSION(:,:),         POINTER::QE
    
END TYPE lutSSP




                    
CONTAINS

    SUBROUTINE nullifyPointers(thisPf, thisRefl, thisSSP,thisFlux, thisAero)
        TYPE (pfData), INTENT(inout), OPTIONAL :: thisPf
        TYPE(lutRefl), INTENT(INOUT), OPTIONAL :: thisRefl
        TYPE(lutSSP),  INTENT(INOUT), OPTIONAL :: thisSSP
        TYPE(lutFlux), INTENT(INOUT), OPTIONAL :: thisFlux
        TYPE(lutAero), INTENT(INOUT), OPTIONAL :: thisAero
        
        IF(PRESENT(thisPf))THEN
           NULLIFY(thisPf%liqpf,thisPf%icepf,thisPf%liqsa, thisPf%icesa)
        ENDIF
        
        IF(PRESENT(thisRefl))THEN
            NULLIFY(thisRefl%msr,thisRefl%solzen,thisRefl%senzen,thisRefl%relazm)
            ! IF(ASSOCIATED(thisRefl%idsol))NULLIFY(thisRefl%idsol)
            !IF(ASSOCIATED(thisRefl%idzen))NULLIFY(thisRefl%idzen)
            !IF(ASSOCIATED(thisRefl%idazm))NULLIFY(thisRefl%idazm)
           
        ENDIF
        
        IF(PRESENT(thisFlux))THEN
            NULLIFY(thisFlux%rflux,thisFlux%tflux,thisFlux%fluxsol,thisFlux%spalb)
        ENDIF
        
        IF(PRESENT(thisSSP))THEN
            NULLIFY(thisSSP%wvl,thisSSP%re,thisSSP%cot,thisSSP%pfconst,thisSSP%truncf,thisSSP%ssalb,thisSSP%QE)
        ENDIF
        
        IF(PRESENT(thisAero))THEN
            NULLIFY(thisAero%aerog,thisAero%aerossa,thisAero%aerocot,thisAero%raycot)
        ENDIF
    END SUBROUTINE nullifyPointers
    
     SUBROUTINE deallocatePointers(thisPf, thisRefl, thisSSP,thisFlux, thisAero)
        TYPE (pfData), INTENT(INOUT), OPTIONAL :: thisPf
        TYPE(lutRefl), INTENT(INOUT), OPTIONAL :: thisRefl
        TYPE(lutSSP),  INTENT(INOUT), OPTIONAL :: thisSSP
        TYPE(lutFlux), INTENT(INOUT), OPTIONAL :: thisFlux
        TYPE(lutAero), INTENT(INOUT), OPTIONAL :: thisAero
        
        IF(PRESENT(thisPf))THEN
           DEALLOCATE(thisPf%liqpf,thisPf%icepf,thisPf%liqsa, thisPf%icesa)
        ENDIF
        
        IF(PRESENT(thisRefl))THEN
            DEALLOCATE(thisRefl%msr,thisRefl%solzen,thisRefl%senzen,thisRefl%relazm)
            ! IF(ASSOCIATED(thisRefl%idsol))NULLIFY(thisRefl%idsol)
            !IF(ASSOCIATED(thisRefl%idzen))NULLIFY(thisRefl%idzen)
            !IF(ASSOCIATED(thisRefl%idazm))NULLIFY(thisRefl%idazm)
           
        ENDIF
        
        IF(PRESENT(thisFlux))THEN
            DEALLOCATE(thisFlux%rflux,thisFlux%tflux,thisFlux%fluxsol,thisFlux%spalb)
        ENDIF
        
        IF(PRESENT(thisSSP))THEN
            DEALLOCATE(thisSSP%wvl,thisSSP%re,thisSSP%cot,thisSSP%pfconst,thisSSP%truncf,thisSSP%ssalb,thisSSP%QE)
        ENDIF
        
        IF(PRESENT(thisAero))THEN
            DEALLOCATE(thisAero%aerog,thisAero%aerossa,thisAero%aerocot,thisAero%raycot)
        ENDIF
    END SUBROUTINE  deallocatePointers
   
    
    SUBROUTINE pfdata_read(this, pfLUTName)
         TYPE (pfData),   INTENT(inout) :: this
         CHARACTER(len=*),INTENT(IN)    :: pfLUTName
         
         INTEGER::allocateStatus , ierror,retstat, SD_id
         INTEGER:: sds_id,rank, dim_id,hdf_id,dimArray(7),start(7),edges(7),stride(7)
        INTEGER:: attrs,dataType,index,attrIndex,count, nreLiq, nreIce, nwl, nAngLiq, nAngIce
        CHARACTER(150) :: name
        

         
        
	    SD_id=SFSTART(trim(pfLUTName),DFACC_RDONLY)
	    IF(SD_id==-1) THEN
		    write(*,*)'FILE READ ERROR'
			ierror = 1             !file read error
			print *, ' oenning hdf file failed in pfdata_read  program ends, check phase filename and/or path.'
            stop
		    RETURN
	    ENDIF

		!begin execution statements
		index = sfn2index(SD_id,'WaterPhaseFuncVals')
		sds_id = sfselect(SD_id,index)
		!Get the dimension informations
		retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
		retstat=sfendacc(sds_id)	

	
		nAngLiq = dimArray(1)
		nwl     = dimArray(2)
		nreLiq  = dimArray(3)

		index = sfn2index(SD_id,'IcePhaseFuncVals')
		sds_id = sfselect(SD_id,index)
		!Get the dimension informations
		retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
		retstat=sfendacc(sds_id)
	
		nAngIce 	= dimArray(1)
		nreIce		= dimArray(3)

     IF(ASSOCIATED(this%liqpf))DEALLOCATE(this%liqpf)
    IF(ASSOCIATED(this%icepf))DEALLOCATE(this%icepf)
    IF(ASSOCIATED(this%liqsa))DEALLOCATE(this%liqsa)
    IF(ASSOCIATED(this%icesa))DEALLOCATE(this%icesa)

	
	IF ( .not. ASSOCIATED(this%liqpf))THEN
	    ALLOCATE(this%liqpf(nAngLiq,nwl, nreLiq),STAT = allocateStatus )
	ENDIF
	
	CALL GetSDSArray(SD_id,'WaterPhaseFuncVals',this%liqpf)
		    		
	IF ( .not. ASSOCIATED(this%icepf))THEN
	    ALLOCATE(this%icepf(nAngIce,nwl,nreIce),STAT = allocateStatus )
	ENDIF
		
	CALL GetSDSArray(SD_id,'IcePhaseFuncVals',this%icepf)
	
	IF ( .not. ASSOCIATED(this%liqsa))THEN
	    ALLOCATE(this%liqsa(nAngLiq),STAT = allocateStatus )
	ENDIF
	
	CALL GetSDSArray(SD_id,'ScatAnglesWater',this%liqsa)
			
	IF ( .not. ASSOCIATED(this%icesa))THEN
	    ALLOCATE(this%icesa(nAngIce),STAT = allocateStatus )
	ENDIF
	
	CALL GetSDSArray(SD_id,'ScatAnglesIce',this%icesa)
	
	retstat=SFEND(SD_id)
	
         
    RETURN 
             
    END SUBROUTINE pfdata_read
    
    
    SUBROUTINE refl_read(lutName,solarAng,viewAng,azmAng,ierror, thisMSR, thisFlux, thisAero, thisSSP)
        CHARACTER(len=*),INTENT(IN):: lutName
        INTEGER,INTENT(OUT)::ierror
	    REAL,INTENT(IN):: solarAng, viewAng, azmAng
        
         TYPE(lutRefl), INTENT(INOUT), OPTIONAL:: thisMSR

        TYPE(lutFlux), INTENT(INOUT), OPTIONAL:: thisFlux
        TYPE(lutAero), INTENT(INOUT), OPTIONAL:: thisAero
        TYPE(lutSSP),  INTENT(INOUT), OPTIONAL:: thisSSP
        
        
        INTEGER::retstat, SD_id, nre, ntau, nwl, nsol, nview, nphi, nwlflux, nsolrftf,nwlrftf
        INTEGER::index,sds_id,rank,dims(7),dataType,attrs,start(7),edges(7),stride(7)
        CHARACTER(150) :: name,attrName
        INTEGER:: ishi,islow, ivhi,ivlow, iazhi,iazlow, attrIndex,count
	    REAL*8::scaleFactor

        REAL:: solarmu, viewmu
        REAL,ALLOCATABLE::tempRefl(:,:,:,:,:,:)
        
        
        
            stride = 1
            start = 0
            solarmu  = COS(solarAng * RAD_PER_DEG)
	        viewmu   = COS(viewAng  * RAD_PER_DEG)


        
			SD_id=SFSTART(TRIM(lutName),DFACC_RDONLY)
			IF(SD_id==-1) THEN
			    write(*,*)'FILE READ ERROR'
				ierror = 1             !file read error
				print *, ' oenning hdf file failed in refl_read  program ends,check lut filename and/or path.'
                stop
				RETURN
			ENDIF
        
		index = sfn2index(SD_id,'MultiScatBDReflectance')
		sds_id = sfselect(SD_id,index)
		!Get the dimension informations
		retStat = sfginfo(sds_id, name, rank, dims, dataType, attrs)
		retstat=sfendacc(sds_id)
		
		nview	= dims(6); nsol	= dims(5); nphi	= dims(4); ntau= dims(3); nwl = dims(2); nre = dims(1)
		edges(1:rank) = dims(1:rank)
		
!		print*,'# of REs =',nre, ', # of taus = ',ntau, ', # of lambdas =', nwl, ', # of solar angs =', nsol, ', # of view angs =', nview, ', # of rela zm angs =',nphi

		IF(PRESENT(thisMSR))THEN
         	    
           IF(ASSOCIATED(thisMSR%solzen))DEALLOCATE(thisMSR%solzen)

		    IF(ASSOCIATED(thisMSR%msr))DEALLOCATE(thisMSR%msr)
           IF(ASSOCIATED(thisMSR%senzen))DEALLOCATE(thisMSR%senzen)
           IF(ASSOCIATED(thisMSR%relazm))DEALLOCATE(thisMSR%relazm)
           ! IF(ASSOCIATED(thisMSR%idsol))DEALLOCATE(thisMSR%idsol)
          ! IF(ASSOCIATED(thisMSR%idzen))DEALLOCATE(thisMSR%idzen)
          ! IF(ASSOCIATED(thisMSR%idazm))DEALLOCATE(thisMSR%idazm)
          
            allocate(thisMSR%solzen(nsol))
            CALL GetSDSArray(SD_id,'ReflectanceSolarZenith',thisMSR%solzen)
            allocate(thisMSR%senzen(nview))
            CALL GetSDSArray(SD_id,'ReflectanceSensorZenith',thisMSR%senzen)
            allocate(thisMSR%relazm(nphi))
			CALL GetSDSArray(SD_id,'ReflectanceRelativeAzimuth',thisMSR%relazm)	


	        CALL BisectionSearch(solarmu,thisMSR%solzen,nsol,islow,ishi)
!print*,islow,ishi
	     !   allocate(thisMSR%idsol(2))
	        thisMSR%idsol = (/islow, ishi/)
	
	        CALL BisectionSearch(viewmu,thisMSR%senzen,nview,ivlow,ivhi)
	     !   allocate(thisMSR%idzen(2))	        
		    thisMSR%idzen = (/ivlow, ivhi/)
!            print*,ivlow,ivhi
	        CALL BisectionSearch(azmang,thisMSR%relazm,nphi,iazlow,iazhi)
	    !    allocate(thisMSR%idazm(2))	        
	        thisMSR%idazm = (/iazlow, iazhi/)
!	        print*,iazlow,iazhi
	        
	        start(4) = iazlow-1
	        start(5) = islow-1
	        start(6) = ivlow-1
	        edges(4:6) = 2
	        allocate(thisMSR%msr(nre,nwl,ntau,2,2,2))
	        
	       

	        index = sfn2index(SD_id,'MultiScatBDReflectance')
	        sds_id = sfselect(SD_id,index)
	        sds_id = sfselect(SD_id,index)
	        retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank),thisMSR%msr)
	        !Read the Scale-Factor, I don't really need it. Just rtreive it
	        attrIndex = sffattr(sds_id, 'Scale_Factor')
	        retstat = sfgainfo(sds_id, attrIndex, attrName, dataType, count)
            retstat = sfrcatt(sds_id, attrIndex, scaleFactor)
            !print*,scaleFactor
            thisMSR%msr = thisMSR%msr * scaleFactor
	        retstat=sfendacc(sds_id)


        ENDIF		
        
        IF(PRESENT(thisFlux))THEN
            IF(.NOT. PRESENT(thisAero))THEN
		    index = sfn2index(SD_id,'TransmittedFlux')
		    sds_id = sfselect(SD_id,index)
		    !Get the dimension informations
		    retStat = sfginfo(sds_id, name, rank, dims, dataType, attrs)
	        nsolrftf	    = dims(4)
		    nwlrftf         = dims(2)
		    
		    start=0
		    stride=1
		    edges(1:rank) = dims(1:rank)
		    edges(4)=2
		    
            
                       
		    IF(ASSOCIATED(thisFlux%rflux))DEALLOCATE(thisFlux%rflux)
            IF(ASSOCIATED(thisFlux%tflux))DEALLOCATE(thisFlux%tflux)
            IF(ASSOCIATED(thisFlux%fluxsol))DEALLOCATE(thisFlux%fluxsol)
            IF(ASSOCIATED(thisFlux%spalb))DEALLOCATE(thisFlux%spalb)
            
            allocate(thisFlux%fluxsol(nsolrftf))
           CALL GetSDSArray(SD_id,'FluxSolarZenith',thisFlux%fluxsol) 
	        CALL BisectionSearch(viewmu,thisFlux%fluxsol,nsolrftf,islow,ishi)
	        thisFlux%idzenf = (/islow, ishi/)

            start(4)= islow-1
            allocate(thisFlux%rflux(nre,nwlrftf,ntau,2))
	        retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank),thisFlux%rflux)
                where(thisFlux%rflux <= 10.0E-9)
                    thisFlux%rflux = 10.0E-9
                end where

	        CALL BisectionSearch(solarmu,thisFlux%fluxsol,nsolrftf,islow,ishi)
	        thisFlux%idsolf = (/islow, ishi/)
            start(4)= islow-1
   
            allocate(thisFlux%tflux(nre,nwlrftf,ntau,2))
	        retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank),thisFlux%tflux)
                where(thisFlux%tflux <= 10.0E-9)
                    thisFlux%tflux = 10.0E-9
                end where
             retstat=sfendacc(sds_id)
    
            
            allocate(thisFlux%spalb(ntau,nwlrftf,nre))            
	        CALL GetSDSArray(SD_id,'SphericalAlbedo',thisFlux%spalb)
           ENDIF
        ENDIF

        IF(PRESENT(thisAero))THEN
        
        IF(ASSOCIATED(thisAero%aerog))DEALLOCATE(thisAero%aerog)
        IF(ASSOCIATED(thisAero%aerossa))DEALLOCATE(thisAero%aerossa)
        IF(ASSOCIATED(thisAero%aerocot))DEALLOCATE(thisAero%aerocot)
        IF(ASSOCIATED(thisAero%raycot))DEALLOCATE(thisAero%raycot)
        
        ALLOCATE(thisAero%aerog(nwl))
		CALL GetSDSArray(SD_id,'AerosolAsymParameter',thisAero%aerog)
        ALLOCATE(thisAero%aerossa(nwl))
		CALL GetSDSArray(SD_id,'AerosolSSAlbedo',thisAero%aerossa)
        ALLOCATE(thisAero%aerocot(nwl))
		CALL GetSDSArray(SD_id,'AerosolOpticalThickness',thisAero%aerocot)
        ALLOCATE(thisAero%raycot(nwl))
	    CALL GetSDSArray(SD_id,'RayleighOpticalThickness',thisAero%raycot)        
        
        ENDIF
        
        IF(PRESENT(thisSSP))THEN
        
        IF(ASSOCIATED(thisSSP%wvl))DEALLOCATE(thisSSP%wvl)
        IF(ASSOCIATED(thisSSP%re))DEALLOCATE(thisSSP%re)
        IF(ASSOCIATED(thisSSP%cot))DEALLOCATE(thisSSP%cot)
        IF(ASSOCIATED(thisSSP%pfconst))DEALLOCATE(thisSSP%pfconst)
        IF(ASSOCIATED(thisSSP%truncf))DEALLOCATE(thisSSP%truncf)
        IF(ASSOCIATED(thisSSP%ssalb))DEALLOCATE(thisSSP%ssalb)
        IF(ASSOCIATED(thisSSP%QE))DEALLOCATE(thisSSP%QE)
        
        
        ALLOCATE(thisSSP%wvl(nwl))
        CALL GetSDSArray(SD_id,'Wavelengths',thisSSP%wvl)        

        ALLOCATE(thisSSP%re(nre))
        CALL GetSDSArray(SD_id,'ParticleRadius',thisSSP%re)        
        ALLOCATE(thisSSP%cot(ntau))
        CALL GetSDSArray(SD_id,'OpticalThickness',thisSSP%cot)        
        ALLOCATE(thisSSP%pfconst(nwl,nre))
        CALL GetSDSArray(SD_id,'PhaseFuncNormConstant',thisSSP%pfconst)

        
        ALLOCATE(thisSSP%truncf(nwl,nre))
		CALL GetSDSArray(SD_id,'TruncationFactor',thisSSP%truncf)

        ALLOCATE(thisSSP%ssalb(nwl,nre))
		CALL GetSDSArray(SD_id,'SingleScatterAlbedo',thisSSP%ssalb)
        

        ALLOCATE(thisSSP%QE(nwl,nre))
        CALL GetSDSArray(SD_id,'ExtinctionCoefficient',thisSSP%QE)        
        
        ENDIF
            
        
        retstat=SFEND(SD_id)
    
    
    
    END SUBROUTINE refl_read
    
  
	!********************* START MODULE PROCEDURES FOR GetSDSArray *********************************

	!--------------------------SUBROUTINE Get_1D_SDS------------------------------------------------
	!This is juat a subroutine to get a 1D SDS data given the SD_ID and SDS name. You need to know
	!the dimensions of the SDS array when passing into the subroutine
	!INPUTS:
	!	SD_ID	:	ID from openning the HDF file 
	!	SDSName	:	Name of the SDS name 
	!OUTPUTS:
	!	dataArray	:	SDS data (1-D)
	!-----------------------------------------------------------------------------------------------
	SUBROUTINE Get_1D_SDS(SD_ID, SDSName, dataArray)
	!
	!Declare Argumenst
	INTEGER,INTENT(IN)::SD_ID
	CHARACTER(len=*),INTENT(IN)::SDSName
	REAL,INTENT(OUT)::dataArray(:)
	!
	!Local variables
	INTEGER::index, sds_id,retStat,rank,dataType,attrs,attrIndex,count
	INTEGER::dimArray(7),start(7),edges(7),stride(7)
	REAL*8::scaleFactor
    CHARACTER(100) :: name,attrName

	!begin execution statements
	index = sfn2index(SD_id,trim(SDSName))
	sds_id = sfselect(SD_id,index)
	!Get the dimension informations
	retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
	
	!read the data into an array. Can do an allocation here: will decide later
	start=0
	stride=1
	edges(1:rank) = dimArray(1:rank)
	retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank),dataArray(1:edges(1)))
	!Read the Scale-Factor, I don't really need it. Just rtreive it
	attrIndex = sffattr(sds_id, 'Scale_Factor')
	retstat = sfgainfo(sds_id, attrIndex, attrName, dataType, count)
    retstat = sfrcatt(sds_id, attrIndex, scaleFactor)
    !print*,scaleFactor
    dataArray = dataArray * scaleFactor
	retstat=sfendacc(sds_id)
!		
	END SUBROUTINE Get_1D_SDS
	!-----------------------------------------------------------------------------------------------
	!--------------------------SUBROUTINE Get_2D_SDS------------------------------------------------
	!This is juat a subroutine to get a 2D SDS data given the SD_ID and SDS name. You need to know
	!the dimensions of the SDS array when passing into the subroutine
	!INPUTS:
	!	SD_ID	:	ID from openning the HDF file 
	!	SDSName	:	Name of the SDS name 
	!OUTPUTS:
	!	dataArray	:	SDS data (2-D)
	!-----------------------------------------------------------------------------------------------
	
	SUBROUTINE Get_2D_SDS(SD_ID, SDSName, dataArray)
	!Declare Arguments
	INTEGER,INTENT(IN)::SD_ID
	CHARACTER(len=*),INTENT(IN)::SDSName
	REAL,INTENT(OUT)::dataArray(:,:)
	!Local variables
	INTEGER::index, sds_id,retStat,rank,dataType,attrs,attrIndex,count
	INTEGER::dimArray(7),start(7),edges(7),stride(7)
	REAL*8::scaleFactor
    CHARACTER(100) :: name,attrName

	!begin execution statements
	index = sfn2index(SD_id,trim(SDSName))
	sds_id = sfselect(SD_id,index)
	!Get the dimension informations
	retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
	
	!read the data into an array. Can do an allocation here: will decide later
	start=0
	stride=1
	edges(1:rank) = dimArray(1:rank)
	retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank), &
	                                                         dataArray(1:edges(1),1:edges(2)))
	!Read the Scale-Factor, I don't really need it. Just rtreive it
	attrIndex = sffattr(sds_id, 'Scale_Factor')
	retstat = sfgainfo(sds_id, attrIndex, attrName, dataType, count)
    retstat = sfrcatt(sds_id, attrIndex, scaleFactor)
    !print*,scaleFactor
    dataArray = dataArray * scaleFactor
	retstat=sfendacc(sds_id)	
	END SUBROUTINE Get_2D_SDS	
	!-----------------------------------------------------------------------------------------------
	!--------------------------SUBROUTINE Get_3D_SDS------------------------------------------------
	!This is juat a subroutine to get a 3D SDS data given the SD_ID and SDS name. You need to know
	!the dimensions of the SDS array when passing into the subroutine
	!INPUTS:
	!	SD_ID	:	ID from openning the HDF file 
	!	SDSName	:	Name of the SDS name 
	!OUTPUTS:
	!	dataArray	:	SDS data (3-D)
	!-----------------------------------------------------------------------------------------------
	
	SUBROUTINE Get_3D_SDS(SD_ID, SDSName, dataArray)
	!Declare Arguments
	INTEGER,INTENT(IN)::SD_ID
	CHARACTER(len=*),INTENT(IN)::SDSName
	REAL,INTENT(OUT)::dataArray(:,:,:)
	!Local variables
	INTEGER::index, sds_id,retStat,rank,dataType,attrs,attrIndex,count
	INTEGER::dimArray(7),start(7),edges(7),stride(7)
	REAL*8::scaleFactor
    CHARACTER(100) :: name,attrName

	!Begin execution statements
	index = sfn2index(SD_id,trim(SDSName))
	sds_id = sfselect(SD_id,index)
	!Get the dimension informations
	retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
	
	!read the data into an array. Can do an allocation here: will decide later
	start=0
	stride=1
	edges(1:rank) = dimArray(1:rank)
	retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank),& 
	                                            dataArray(1:edges(1),1:edges(2),1:edges(3)))
	!Read the Scale-Factor, I don't really need it. Just rtreive it
	attrIndex = sffattr(sds_id, 'Scale_Factor')
	retstat = sfgainfo(sds_id, attrIndex, attrName, dataType, count)
    retstat = sfrcatt(sds_id, attrIndex, scaleFactor)
    !print*,scaleFactor
    dataArray = dataArray * scaleFactor
	retstat=sfendacc(sds_id)	
	END SUBROUTINE Get_3D_SDS	
	!-----------------------------------------------------------------------------------------------
	!--------------------------SUBROUTINE Get_4D_SDS------------------------------------------------
	!This is juat a subroutine to get a 4D SDS data given the SD_ID and SDS name. You need to know
	!the dimensions of the SDS array when passing into the subroutine
	!INPUTS:
	!	SD_ID	:	ID from openning the HDF file 
	!	SDSName	:	Name of the SDS name 
	!OUTPUTS:
	!	dataArray	:	SDS data (4-D)
	!-----------------------------------------------------------------------------------------------
	
	SUBROUTINE Get_4D_SDS(SD_ID, SDSName, dataArray)
	!Declare Arguments
	INTEGER,INTENT(IN)::SD_ID
	CHARACTER(len=*),INTENT(IN)::SDSName
	REAL,INTENT(OUT)::dataArray(:,:,:,:)
	!Local variables
	INTEGER::index, sds_id,retStat,rank,dataType,attrs,attrIndex,count
	INTEGER::dimArray(7),start(7),edges(7),stride(7)
	REAL*8::scaleFactor
    CHARACTER(100) :: name,attrName

	!Begin execution statements
	index = sfn2index(SD_id,trim(SDSName))
	sds_id = sfselect(SD_id,index)
	!Get the dimension informations
	retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
	
	!read the data into an array. Can do an allocation here: will decide later
	start=0
	stride=1

	edges(1:rank) = dimArray(1:rank)
	retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank), & 
	                   dataArray(1:edges(1),1:edges(2),1:edges(3),1:edges(4)))
	!Read the Scale-Factor, I don't really need it. Just rtreive it
	attrIndex = sffattr(sds_id, 'Scale_Factor')
	retstat = sfgainfo(sds_id, attrIndex, attrName, dataType, count)
    retstat = sfrcatt(sds_id, attrIndex, scaleFactor)
    !print*,scaleFactor
    dataArray = dataArray * scaleFactor
	retstat=sfendacc(sds_id)	
	END SUBROUTINE Get_4D_SDS
	
	SUBROUTINE Get_5D_SDS(SD_ID, SDSName, dataArray)
	!-----------------------------------------------------------------------------------------------
	!--------------------------SUBROUTINE Get_5D_SDS------------------------------------------------
	!This is juat a subroutine to get a 5D SDS data given the SD_ID and SDS name. You need to know
	!the dimensions of the SDS array when passing into the subroutine
	!INPUTS:
	!	SD_ID	:	ID from openning the HDF file 
	!	SDSName	:	Name of the SDS name 
	!OUTPUTS:
	!	dataArray	:	SDS data (5-D)
	!-----------------------------------------------------------------------------------------------
	!Declare Arguments
	
	INTEGER,INTENT(IN)::SD_ID
	CHARACTER(len=*),INTENT(IN)::SDSName
	REAL,INTENT(OUT)::dataArray(:,:,:,:,:)
	
	!Local variables
	INTEGER::index, sds_id,retStat,rank,dataType,attrs,attrIndex,count
	INTEGER::dimArray(7),start(7),edges(7),stride(7)
	REAL*8::scaleFactor
    CHARACTER(100) :: name,attrName

	
	!begin execution
	index = sfn2index(SD_id,trim(SDSName))
	sds_id = sfselect(SD_id,index)
	!Get the dimension informations
	retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
	
	!read the data into an array. Can do an allocation here: will decide later
	start=0
	stride=1
	edges(1:rank) = dimArray(1:rank)
	retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank), & 
	                  dataArray(1:edges(1),1:edges(2),1:edges(3),1:edges(4),1:edges(5)))
	!Read the Scale-Factor, I don't really need it. Just rtreive it
	attrIndex = sffattr(sds_id, 'Scale_Factor')
	retstat = sfgainfo(sds_id, attrIndex, attrName, dataType, count)
    retstat = sfrcatt(sds_id, attrIndex, scaleFactor)
    !print*,scaleFactor
    dataArray = dataArray * scaleFactor
	retstat=sfendacc(sds_id)	
	END SUBROUTINE Get_5D_SDS	
	!-----------------------------------------------------------------------------------------------
	!--------------------------SUBROUTINE Get_6D_SDS------------------------------------------------
	!This is juat a subroutine to get a 6D SDS data given the SD_ID and SDS name. You need to know
	!the dimensions of the SDS array when passing into the subroutine
	!INPUTS:
	!	SD_ID	:	ID from openning the HDF file 
	!	SDSName	:	Name of the SDS name 
	!OUTPUTS:
	!	dataArray	:	SDS data (6-D)
	!-----------------------------------------------------------------------------------------------
	
	SUBROUTINE Get_6D_SDS(SD_ID, SDSName, dataArray)
	
	INTEGER,INTENT(IN)::SD_ID
	CHARACTER(len=*),INTENT(IN)::SDSName
	REAL,INTENT(OUT)::dataArray(:,:,:,:,:,:)
	
	INTEGER::index, sds_id,retStat,rank,dataType,attrs,attrIndex,count
	INTEGER::dimArray(7),start(7),edges(7),stride(7)
	REAL*8::scaleFactor
    CHARACTER(100) :: name,attrName

	
	index = sfn2index(SD_id,trim(SDSName))
	sds_id = sfselect(SD_id,index)
	!Get the dimension informations
	retStat = sfginfo(sds_id, name, rank, dimArray, dataType, attrs)
	
	!read the data into an array. Can do an allocation here: will decide later
	start=0
	stride=1
	edges(1:rank) = dimArray(1:rank)
	retstat = sfrdata(sds_id,start(1:rank),stride(1:rank),edges(1:rank), & 
	                 			dataArray(1:edges(1),1:edges(2),1:edges(3),1:edges(4), &
	                 												1:edges(5),1:edges(6)))
	!Read the Scale-Factor, I don't really need it. Just rtreive it
	attrIndex = sffattr(sds_id, 'Scale_Factor')
	retstat = sfgainfo(sds_id, attrIndex, attrName, dataType, count)
    retstat = sfrcatt(sds_id, attrIndex, scaleFactor)
    !print*,scaleFactor
    dataArray = dataArray * scaleFactor
	retstat=sfendacc(sds_id)	
	END SUBROUTINE Get_6D_SDS
	!-----------------------------------------------------------------------------------------------
	!********************* END MODULE PROCEDURES FOR GetSDSArray ***********************************
	

	!-------------------------FUNCTION ScatAngle ---------------------------------------------------
	!This function computes the scattering angle for a given sun-satellite geometry
	!INPUTS:
	!	solarAng : solar angle in degrees (scalar)
	!	viewAng	 : view angle in degrees (scalar)
	!	relAzm	 : rel azimuth (from the principle plane of the SUN) (scalar)
	!OUTPUTS:
	!	scatAngle : scattering angle in degrees (scalar)
	!-----------------------------------------------------------------------------------------------
	REAL FUNCTION ScatAngle(solarAng,viewAng,relAzm)
	IMPLICIT NONE
	!CALLING ARGUMENTS
	REAL, INTENT(IN):: solarAng,viewAng,relAzm

	!LOCAL VARIABLES
	REAL::solarmu,viewmu,pi,torad,todeg


	pi		= acos(-1.d0 )
	torad	= pi / 180.d0
	todeg	= 180.d0 / pi

	solarmu = COS(solarAng * torad)
	viewmu = COS(viewAng * torad)

	scatAngle= -solarmu*viewmu + SQRT( ( 1.- solarmu**2 )* &
                      ( 1.- viewmu**2 ) )*COS( relAzm*torad )
    IF(scatAngle < -1.0)scatAngle = -1.0 !guard against round off error 
    IF(scatAngle > 1.0)scatAngle = 1.0
    	
                     
	scatAngle = ACOS(scatAngle)* todeg

	END FUNCTION ScatAngle
	!-----------------------------------------------------------------------------------------------
	
	!-------------------------- SUBROUTINE getAllPhaseFunctionValues ----------------------------------
	!This program interpolates the phase fun value array for a given scattering angle an returns
	!the corresponding values for particular lambda and re (see the inputs and outputs)
	!eventhough it's much simpler to calculate Rayleigh and aerosol PF values, put evrything into
	!this routine because of passage of scattering angle and they fall into similar category
	!INPUTS:
	!	scatAngle		:	scattering angle in degrees (scalar)
	!	scatAngleArray	:	1-D array of sacattering angles for which phase fun vals are precomputed
	!	phaseFunArray	:	3-D array of phase fun vals: 1st DIM = num of angles, 
	!						2nd DIM = num of wavelengths, 3rd DIM = num of REs
	!	phaseNormConst	:	Normalization factor for the phase function. If it is normalized to 1
	!						then it is OK. But, this number (= pmom(0)) from the dfit routines  has
	!						been saved and included in the library file in adition to d_fit coeffs.
	!	aeroG		    :	aerosol asymmetry parameter
	!OUTPUTS:
	!	phaseFunVals	:	2-D array of interpolated phase fun values at scatAngle
	!						1st dim = num of wavelengths, 2nd dim = num of rE's
	!	RLphase			:	rayleigh phase function value (scalar)
	!	aeroPhase		:	aerosol phase function value (1-d vector)
	!	ierror			:	integer value
	!						ierror 	= 0  	success, 
	!								= 3		invalid data
	!-----------------------------------------------------------------------------------------------
	! 						
	SUBROUTINE GetAllPhaseFunctionValues(scatAngle, scatAngleArray, phaseFunArray, phaseNormConst, & 
	                                          phaseFunVals,ierror,aeroG,RLphase,aeroPhase)
	IMPLICIT NONE
	!Arguments
	REAL,INTENT(IN)::scatAngle, scatAngleArray(:),phaseFunArray(:,:,:),phaseNormConst(:,:)
	REAL,INTENT(IN), OPTIONAL::aeroG(:)
	REAL,INTENT(OUT)::phaseFunVals(:,:)
	REAL,INTENT(OUT), OPTIONAL::RLphase,aeroPhase(:)
	INTEGER,INTENT(OUT)::ierror

	!local variables
	INTEGER:: iAng, iAngLow, iAngHi,nwl,allocateStatus
	REAL::dtheta,cosScatAng
	REAL, ALLOCATABLE :: gsquare(:)
	CHARACTER(150)::tempstr

	!start execution
	
	
	ierror = 0 !start with no error
	iAngLow = 1
	iAngHi = size(scatAngleArray)
	IF(scatAngle > scatAngleArray(iAngHi))THEN
		iAngLow = iAngHi-1 !no need to do the search; do extrapolation
        phaseFunVals(:,:) =  phaseFunArray(iAngHi,:,:)/phaseNormConst(:,:)
                !phaseFunVals(:,:) =  phaseFunArray(iAngHi-4,:,:)/phaseNormConst(:,:)


	    RETURN
		
		
	ELSEIF(scatAngle < scatAngleArray(iAngLow))THEN
		iAngHi = iAngLow+1
	ELSE
		DO									!do a bisection search
			IF((iAngHi - iAngLow) == 1)EXIT
			iAng = (iAngHi + iAngLow)/2
			IF(scatAngleArray(iAng) < scatAngle)THEN
			iAngLow = iAng
			ELSE
				iAngHi = iAng
			ENDIF
		ENDDO
	ENDIF
	
	if(PRESENT(aeroG))THEN
	nwl = size(aeroG)
	ALLOCATE(gsquare(1:nwl),STAT = allocateStatus)
				
	IF(allocateStatus /=0) THEN
		ierror = 2			!memory allocation error
		RETURN
		ENDIF			 	

	
	cosScatAng = COS(scatAngle* RAD_PER_DEG)
	gsquare = aerog*aerog
	
	RLphase = 3.0 * (1 + (cosScatAng)**2)/4.0
	aeroPhase = 0.0
	aeroPhase(:) = (1.0 - gsquare)/(sqrt(1 + gsquare - 2.0 * aerog * cosScatAng))**3
	aeroPhase(nwl-1:nwl) = 0.0
	
	DEALLOCATE(gsquare)
    ENDIF
    
	dtheta = scatAngleArray(iAngHi) - scatAngleArray(iAngLow)
	IF(dtheta ==0)THEN
		ierror = 3 			!invalid data read
		RETURN
	ENDIF
	

	phaseFunVals(:,:) = ( phaseFunArray(iAngLow,:,:)  * (scatAngleArray(iAngHi)-scatAngle) + &
                    phaseFunArray(iAngHi,:,:)  * (scatAngle - scatAngleArray(iAngLow) ))/dtheta 
    phaseFunVals(:,:) = phaseFunVals(:,:)/phaseNormConst(:,:)

	RETURN

	END SUBROUTINE GetAllPhaseFunctionValues
	!-----------------------------------------------------------------------------------------------
	!-------------------------- SUBROUTINE InterpolateFluxes ---------------------------------------
	!This program interpolates(linear) the RF or TF array for a given solar/oview angle and returns
	!the corresponding values for particular tau, lambda and re (see the inputs and outputs)
	!INPUTS:
	!	solarOrViewAng		:	input angle in degrees (scalar)
	!	solarAngleMuArray	:	1-D array of solar angles for which RF/TF vals are precomputed
	!	inFluxArray			:	4-D array of flux vals: 1st DIM = num of angles, 2nd DIM = ntaus
	!						 	3rdd DIM = num of wavelengths, 4th DIM = num of REs
	!OUTPUTS:
	!	outFluxArray		: 3-D array of interpolated RF or TF values at the input angle
	!						  1st dim = ntaus, 2nd dim = num of wavelengths, 3rdd dim = num of rE's
	!	ierror			:	integer value
	!						ierror 	= 0  	success, 
	!								= 3		invalid data
	!-----------------------------------------------------------------------------------------------
	! 						
	SUBROUTINE InterpolateFluxes(solarOrViewAng, solarAngMuArray, inFluxArray, outFluxArray, ierror)
	IMPLICIT NONE
	!Arguments
	INTEGER,INTENT(IN):: solarOrViewAng(2)
	REAL,INTENT(IN)::solarAngMuArray(:),inFluxArray(:,:,:,:)
	REAL,INTENT(OUT)::outFluxArray(:,:,:)
	INTEGER,INTENT(OUT)::ierror

	!local variables
	INTEGER:: iAng, iAngLow, iAngHi
	REAL*8::dtheta,UMU
	CHARACTER(100)::tempstr

	ierror = 0
	
	iAngLow = solarOrViewAng(1)
	iAngHi = solarOrViewAng(2)

	dtheta = solarAngMuArray(iAngHi) - solarAngMuArray(iAngLow)
	IF(dtheta ==0)THEN
		ierror = 3    !invalid data read
		RETURN
	ENDIF


    
	outFluxArray(:,:,:) = ( inFluxArray(:,:,:,1)  * (solarAngMuArray(iAngHi)- UMU) + &
                    inFluxArray(:,:,:,2)  * (UMU - solarAngMuArray(iAngLow) ))/dtheta 
    where(outFluxArray <= 10.0E-10)
        outFluxArray = 10.0E-10
    end where
    RETURN
  	outFluxArray(:,:,:) =  inFluxArray(:,:,:,iAngLow)  + &
                    ( inFluxArray(:,:,:,iAngHi) - inFluxArray(:,:,:,iAngLow) )  *  & 
                    (UMU - solarAngMuArray(iAngLow) )/dtheta                   

	RETURN

	END SUBROUTINE InterpolateFluxes
	!-----------------------------------------------------------------------------------------------
	!-----------------------------------------------------------------------------------------------
	!----------------------SUBROUTINE CalcSingScatPartOfReflectanceNew------------------------------
	!Calculates the single scattering part of the reflectance according to Nakajima/Tanaka method
	!implemented in DISORT Version2. We had to implement this way, because of the way single 
	!scattering part is takent out in DISORT. Please refer to the DISORT2 documentation
	!Formula Used here: refl = omega * PC * {1.0 - EXP[-tauc *(1/mu + 1/mu0)]}
	!                   refl = refl/(4.0 * (mu + mu0))
	!				    where tauc = tau * [Qe(lambda,re)/Qe(CH1,re)] * (1.0 - ftrunc * omega)
	!						   PC  = Ptrue / (1 - ftrunc * omega)
	!					also remember refl_lUT = (I_disort) * pi/( mu0* solarflux)
	!					so that multiplication is already taken care of here, no need to do it
	!					outside of this routine
	!INPUTS:
	!	pf           : derived type for phase function data
	!   ss           : derived type for single scattering properies & RE aray, tau array
	!   viewang      : view angle in degrees
	!   solarang     : solar angle in degrees
	!   azmang       : relative azimuth in degrees
	!   phase        : 'ICE' or 'LIQUID', string
	!OPTIONAL INPUTS:
	!   surfType     : syrface type, lambertian or ocean
	!   as           : derived type for aerosol & Rayleigh data (for ocean)	
	!   
	!OUTPUTS:
	!   ssRefl       : single scattering part of the reflectance , here no need to input the beam
	!                 strength, since we calculate the reflectance.
	!	ierror			:	integer value
	!						ierror 	= 0  	success, 
	!								= 2 	memory allocation error
	!								= 3		invalid data
	!-----------------------------------------------------------------------------------------------	
	SUBROUTINE CalcSingScatPartOfReflectanceNew(pf, ss, solarang, viewang, azmang, phase, ssRefl, &
	                                            ierror, surfType, as)
	IMPLICIT NONE
	
    TYPE(pfData),INTENT(IN)::pf
    TYPE(lutSSP),INTENT(IN)::ss
    REAL, INTENT(IN)::solarang,viewang,azmang

	REAL, INTENT(OUT) :: ssRefl(:,:,:)
    CHARACTER(len=*),INTENT(IN):: phase
    INtEGER,INTENT(OUT)::ierror
    CHARACTER(len=*),INTENT(IN), OPTIONAL:: surfType
    TYPE(lutAero),INTENT(IN),OPTIONAL::as
	
	
	INTEGER:: ntau, nwl, nre, itau,iwl,ire, allocateStatus
	REAL ::  UMU, UMU0, MUPLUSMU0,ratio,temparg,scat_angle
	REAL, ALLOCATABLE ::  fOmega(:,:),fQe(:,:),fPhase(:,:),tauC(:,:),phaseFunVals(:,:),tauLR(:,:), &
	                      EXP0(:,:),EXP1(:,:), aeroPhase(:),aeroTau(:),aeroOmega(:)
	REAL::tempPhase, tempOmega,RLphase
	


	ierror = 0
	ntau = size(ss%cot); nwl = size(ss%ssalb,1); nre = size(ss%ssalb,2)
	
	ALLOCATE(fOmega(1:nwl,1:nre),fQe(1:nwl,1:nre),fPhase(1:nwl,1:nre),tauC(1:nre,1:nwl), &
				phaseFunVals(1:nwl,1:nre),tauLR(1:nre,1:nwl),EXP0(1:nre,1:nwl),EXP1(1:nre,1:nwl), &
				aeroPhase(1:nwl),aeroTau(1:nwl),aeroOmega(1:nwl),STAT = allocateStatus)
				
	IF(allocateStatus /=0) THEN
		ierror = 2			!memory allocation error
		RETURN
		ENDIF			 	
			 	
	scat_angle = scatAngle(solarang,viewang,azmang)
	IF(PRESENT(as))THEN
	    IF(trim(phase) == 'LIQUID' .OR. trim(phase) == 'liquid' )then
	        CALL GetAllPhaseFunctionValues(scat_angle, pf%liqsa, pf%liqpf, & 
		                             ss%pfconst,phaseFunVals,ierror,as%aerog,RLphase,aeroPhase)	
        ENDIF
	    IF(trim(phase) == 'ICE' .OR. trim(phase) == 'ice')then
	        CALL GetAllPhaseFunctionValues(scat_angle, pf%icesa, pf%icepf, & 
		                             ss%pfconst,phaseFunVals,ierror,as%aerog,RLphase,aeroPhase)	
        ENDIF
    ELSE
	    IF(trim(phase) == 'LIQUID' .OR. trim(phase) == 'liquid' )then
	        CALL GetAllPhaseFunctionValues(scat_angle, pf%liqsa, pf%liqpf, & 
		                             ss%pfconst,phaseFunVals,ierror)	
        ENDIF
	    IF(trim(phase) == 'ICE' .OR. trim(phase) == 'ice')then
	        CALL GetAllPhaseFunctionValues(scat_angle, pf%icesa, pf%icepf, & 
		                             ss%pfconst,phaseFunVals,ierror)	
        ENDIF
    ENDIF
    
    
    

	UMU  = COS(viewang * RAD_PER_DEG)
	UMU0 = COS(solarang * RAD_PER_DEG)
	
	IF(UMU ==0.0 .OR. UMU0 == 0.0)THEN
		ierror = 3	!invalida data error
		RETURN
	ENDIF
	
	MUPLUSMU0 = UMU + UMU0
	ratio = MUPLUSMU0/(UMU * UMU0)
	
	fQe(1,:) = 1.0
	do iwl = 2,nwl
		fQe(iwl,:) = ss%QE(iwl,:)/ss%QE(1,:)
	enddo
	
	fOmega = ss%truncf * ss%ssalb             !ssalbedo * ftrunc= (omega*f)
	fPhase = phaseFunVals/ (1.0 - fOmega)   !add something here to safeguard div by zero
	
	IF(PRESENT(as))THEN
	        aeroTau = as%aerocot
	        aeroOmega = as%aerossa
	        ssRefl = 0.0
	        DO itau = 1, ntau

		        tauLR =  ss%cot(itau) * transpose(fQe) *(1.0 - transpose(fOmega))

		        EXP0 = 1.0
		        EXP1 = 0.0
		
		        where(tauLR*ratio < EXP_TOL)
		            EXP1= EXP(-tauLR * ratio)
		        end where
		
		
		
		        ssRefl(:,:,itau) = ssRefl(:,:,itau) + transpose(ss%ssalb) * transpose(fPhase) * & 
		                            (EXP0 - EXP1)
		
		        EXP0 = EXP1
		        DO iwl = 1, nwl-2
			        tauLR(:,iwl) =  tauLR(:,iwl) + as%raycot(iwl)	
			        EXP1(:,iwl) = 0.0
			        where(tauLR(:,iwl) * ratio < EXP_TOL)
			            EXP1(:,iwl) = EXP(-tauLR(:,iwl) * ratio)
			        end where
			
			        ssRefl(:,iwl,itau) = ssRefl(:,iwl,itau) +  & 
			                            RLphase * 1.0 * (EXP0(:,iwl) - EXP1(:,iwl))
			
			        EXP0(:,iwl) = EXP1(:,iwl)
			
			        tempOmega = ((aeroTau(iwl) -0.1) * 1.0 + 0.1 * aeroOmega(iwl))/aeroTau(iwl)

			        tauLR(:,iwl) = (tauLR(:,iwl)  + aeroTau(iwl)) 
			        !need to take care of mixin rayleigh molecules with aerosol 
			        !(that's what is done in DISORT)
			        tempPhase = ((aeroTau(iwl) -0.1) * RLphase * 1.0 + & 
			               0.1 * aeroOmega(iwl) * aeroPhase(iwl))/ &
			                        ( (aeroTau(iwl) -0.1)*1.0 + 0.1 * aeroOmega(iwl) )
			
			        !coming in aeroTau is the (rayleightau + 0.1) from DISORT run
			        EXP1(:,iwl)=0.0
			        where(tauLR(:,iwl) * ratio < EXP_TOL)
			            EXP1(:,iwl) = EXP(-tauLR(:,iwl) * ratio)
			        end where
			        ssRefl(:,iwl,itau) = ssRefl(:,iwl,itau) + tempPhase * tempOmega * &
			                                                (EXP0(:,iwl) - EXP1(:,iwl))
		        ENDDO

				 
	        ENDDO
	    
	ELSE
	ssRefl = 0.0
	do itau = 1, ntau
		tauC = (ss%cot(itau) * transpose(fQe))    !Qe sacling
		tauC = tauC * (1.0 - transpose(fOmega))   !tau scaling, phase func truncation
		tauC = tauC * ratio       ! tau *(1/mu + 1/mu0)

	    where(tauC < EXP_TOL)
		    ssRefl(:,:,itau) = transpose(ss%ssalb) * transpose(fPhase) * (1 - EXP(-tauC))  
		                          ! omega * P * {1-exp[- tau *(1/mu + 1/mu0)]}
		elsewhere
		    ssRefl(:,:,itau) = transpose(ss%ssalb) * transpose(fPhase)
		end where
		
	enddo
	ENDIF
	
	ssRefl = ssRefl/(4.0 * MUPLUSMU0)
	
	
	DEALLOCATE(aeroOmega, aeroTau, aeroPhase,EXP1,EXP0,tauLR,phaseFunVals,tauC,fPhase,fQe,fOmega)
	RETURN
	
	END SUBROUTINE CalcSingScatPartOfReflectanceNew			
	!-----------------------------------------------------------------------------------------------
	!--------------------------SUBROUTINE GetReflForGivenSurfAlbedoWaterNew----------------------------
	!This subroutine outouts the Reflectance table (R(tau,lambda, re)) for a given sun-satellite
	!geometry and a surface albedo
	!INPUTS	:
	!   pf          : structure with phase function data
	!   ss           : derived type for single scattering properies & RE aray, tau array
	!   lut         : MS reflectance daya, derived type
	!   flx         : derived type for flux data
	!	solarAng	:	solar zenith in degrees (scalar)
	!	viewAng		:	viewing angle in degrees (scalar)
	!	azmAng		:	relative azimuth between sat and solar planes
	!	surfAlb		:	surface albedo
	!OUTPUTS :
	!	reflAsurf	:	reflectance as a function of tau,lambda,re : 3-D Array
	!					DIM-1 = ntau, DIM-2 = nlambda, DIM-3 = nre
	!	ierror			:	integer value
	!						ierror 	= 0  	success, 
	!								= 1 	I/O error
	!								= 2 	memory allocation error
	!								= 3		invalid data	
	!ROUTINES CAlLED :
	!	ReadPhaseFunctionDataHDF
	!	ReadWaterLibraryHDF	
	!	GetPhaseFunctionValues
	!	TriLinearInterpolationReflSearch
	!	CalcSingScatPartOfReflectance
	!	InterpolateFluxes
	!-----------------------------------------------------------------------------------------------
	SUBROUTINE GetReflForGivenSurfAlbedoNew(pf,ss,lut,flx,solarAng,viewAng,azmAng,surfAlb,phase, & 
	                                         reflAsurf,ierror)
	IMPLICIT NONE
    TYPE(pfData),INTENT(IN)::pf
    TYPE(lutSSP),INTENT(IN)::ss
    TYPE(lutRefl),INTENT(IN)::lut
    TYPE(lutFlux),INTENT(IN)::flx
    
    REAL, INTENT(IN)::solarang,viewang,azmang,surfAlb

	REAL, INTENT(OUT) :: reflAsurf(:,:,:)
    CHARACTER(len=*),INTENT(IN):: phase
	INTEGER,INTENT(OUT)::ierror
	
	INTEGER::allocateStatus,ire,iwl,nre,ntau,nwl
	REAL :: scat_angle
	REAL,ALLOCATABLE:: multiScatRefl(:,:,:),ssRefl(:,:,:),&
										outTFluxArrayMu(:,:,:),outTFluxArrayMu0(:,:,:)
	
	
	
	ntau = size(ss%cot); nwl = size(ss%ssalb,1); nre = size(ss%ssalb,2)
	
	scat_angle = scatAngle(solarAng,viewAng,azmAng)
	
	
	reflAsurf = -9999.0
	
	ALLOCATE(multiScatRefl(1:nre,1:nwl,1:ntau), &
	     	ssRefl(1:nre,1:nwl,1:ntau), &
	     	outTFluxArrayMu(1:nre,1:nwl,1:ntau), &
		   	outTFluxArrayMu0(1:nre,1:nwl,1:ntau), STAT = allocateStatus)
			 	
	IF(allocateStatus /=0) THEN
		ierror = 2			!memory allocation error
		RETURN
	ENDIF			 	
			 

	!interpolate multi scattering part of the reflection
	CALL TriLinearInterpolationRefl(solarAng,viewAng,azmAng, lut,multiScatRefl,ierror)	
	IF(ierror /= 0)RETURN      !if there is any error no point of continuing                        
	                                    
    CALL CalcSingScatPartOfReflectanceNew(pf,ss, solarAng,viewAng,azmAng, phase, ssRefl, ierror)
								 
	multiScatRefl = ssRefl + multiScatRefl  !total reflection	with asurf = 0.0	

	!interpolate fluxes
	CALL InterpolateFluxes(flx%idsolf, flx%fluxsol, flx%tflux, outTFluxArrayMu0,ierror)
	IF(ierror /= 0)RETURN      !if there is any error no point of continuing                        

	CALL InterpolateFluxes(flx%idzenf, flx%fluxsol, flx%rflux, outTFluxArrayMu,ierror)
	IF(ierror /= 0)RETURN      !if there is any error no point of continuing                        

	!get the reflectance for non zero surf albedo (add the contribution from a/=0)
	do iwl=1, nwl-1
	do ire = 1, nre
	    reflAsurf(:,iwl,ire) = multiScatRefl(ire,iwl,:) + &
     (surfAlb * outTFluxArrayMu0(ire,iwl,:) * outTFluxArrayMU(ire,iwl,:)) / ( 1.0 - surfAlb * flx%spalb(:,iwl,ire))
    enddo
    enddo
	
	DEALLOCATE(outTFluxArrayMu0, outTFluxArrayMu, ssRefl, multiScatRefl)
	RETURN
	END SUBROUTINE GetReflForGivenSurfAlbedoNew
	!-----------------------------------------------------------------------------------------------
	
	!--------------------------SUBROUTINE GetReflForGivenWindSpeedNew-----------------------------
	!This subroutine outputs the Reflectance table (R(tau,lambda, re)) for a given sun-satellite
	!geometry and a wind speed
	!INPUTS	:
	!   pf          : structure with phase function data
	!   ss          : derived type for single scattering properies & RE aray, tau array
	!   lutArray    : MS ocean reflectance data, derived type
	!   as          : derived type for aerosol, rayleigh properties
	!   lut_wspeeds : wind speeds corrresponds to ocean LUTs
	!	solarAng	:	solar zenith in degrees (scalar)
	!	viewAng		:	viewing angle in degrees (scalar)
	!	azmAng		:	relative azimuth between sat and solar planes (scalar)
	!	wspeed		:	wind speed m/s (scalar)
	!OUTPUTS :
	!	reflAsurf	:	reflectance as a function of tau,lambda,re : 3-D Array
	!					DIM-1 = ntau, DIM-2 = nlambda, DIM-3 = nre
	!	ierror			:	integer value
	!						ierror 	= 0  	success, 
	!								= 1 	I/O error
	!								= 2 	memory allocation error
	!								= 3		invalid data	
	!ROUTINES CAlLED :
	!	TriLinearInterpolationReflSearch
	!	CalcSingScatPartOfReflectanceNew
	!-----------------------------------------------------------------------------------------------
	SUBROUTINE GetReflForGivenWindSpeedNew(pf,ss,lutArray,as,lut_wspeeds,solarAng,viewAng,azmAng, & 
	                                        wspeed,phase,reflAsurf,ierror)
	IMPLICIT NONE
    REAL,INTENT(IN)::lut_wspeeds(:)
    TYPE(pfData),INTENT(IN)::pf
    TYPE(lutSSP),INTENT(IN)::ss
    TYPE(lutRefl),INTENT(IN)::lutArray(size(lut_wspeeds))
    TYPE(lutAero),INTENT(IN)::as
    
	REAL, INTENT(IN) :: solarAng,viewAng,azmAng,wspeed
    CHARACTER(len=*),INTENT(IN):: phase

	   															
	!
	REAL,INTENT(OUT) :: reflAsurf(:,:,:)
	INTEGER,INTENT(OUT)::ierror
	
	INTEGER::allocateStatus,iuhi,iulow, nre, ntau, nwl,nwspeed,iwl,ire
	REAL :: scat_angle,deltau
	REAL,ALLOCATABLE::multiScatRefl(:,:,:), multiScatRefl_1(:,:,:),multiScatRefl_2(:,:,:), &
	   				ssRefl(:,:,:)
										
	
	
	ntau = size(ss%cot); nwl = size(ss%ssalb,1); nre = size(ss%ssalb,2);nwspeed = size(lut_wspeeds)
	
!	print*,size(reflAsurf,1);print*,size(reflAsurf,2);print*,size(reflAsurf,3)
	reflAsurf = -9999.0

	scat_angle = scatAngle(solarAng,viewAng,azmAng)
	
	ALLOCATE(multiScatRefl(1:nre,1:nwl,1:ntau), &
			 ssRefl(1:nre,1:nwl,1:ntau), STAT = allocateStatus)
    CALL CalcSingScatPartOfReflectanceNew(pf, ss, solarang, viewang, azmang, phase, ssRefl,ierror, &
                                           'OCEAN', as)	
		 	
			 	
	IF(allocateStatus /=0) THEN
		ierror = 2			!memory allocation error
		RETURN
	ENDIF			 	
			 
	                                    	
		
	IF( (wspeed - lut_wspeeds(1))  <= 0.2)THEN

		!interpolate multi scattering part of the reflection
	 CALL TriLinearInterpolationRefl(solarAng,viewAng,azmAng, lutArray(1),multiScatRefl,ierror)	
		IF(ierror /= 0)RETURN      !if there is any error no point of continuing      


		multiScatRefl = ssRefl + multiScatRefl  !total reflection	with wspeed = 3.0
		
	ELSEIF( (wspeed - lut_wspeeds(nwspeed)) >=  -0.2)THEN

	 CALL TriLinearInterpolationRefl(solarAng,viewAng,azmAng, lutArray(nwspeed),multiScatRefl,ierror)	
		IF(ierror /= 0)RETURN      !if there is any error no point of continuing      


		multiScatRefl = ssRefl + multiScatRefl !total reflection	with wspeed = 15.0	

		
	ELSE
	
	    CALL BisectionSearch(wspeed, lut_wspeeds, nwspeed,iulow,iuhi)

		ALLOCATE(multiScatRefl_1(1:nre,1:nwl,1:ntau),multiScatRefl_2(1:nre,1:nwl,1:ntau), &
					STAT = allocateStatus)
	
		IF(allocateStatus /=0) THEN
			ierror = 2			!memory allocation error
			RETURN
		ENDIF			 	

	 CALL TriLinearInterpolationRefl(solarAng,viewAng,azmAng, lutArray(iulow), multiScatRefl_1,ierror)	

	 CALL TriLinearInterpolationRefl(solarAng,viewAng,azmAng, lutArray(iuhi), multiScatRefl_2,ierror)	


        deltau = lut_wspeeds(iuhi) - lut_wspeeds(iulow)
		multiScatRefl(:,:,:) = ( multiScatRefl_1  * (lut_wspeeds(iuhi)- wspeed) + &
                    		multiScatRefl_2  * (wspeed - lut_wspeeds(iulow) ))/deltau	
        multiScatRefl = multiScatRefl + ssRefl
        
		DEALLOCATE(multiScatRefl_2,multiScatRefl_1)
	ENDIF
	
	do iwl=1, nwl-1
	do ire = 1, nre
	    reflAsurf(:,iwl,ire) = multiScatRefl(ire,iwl,:) 
    enddo
    enddo
	
	DEALLOCATE(ssRefl,multiScatRefl)
	RETURN
	
		CONTAINS

		SUBROUTINE BisectionSearch(theta,thetaArray,ntheta,iAngLow, iAngHi)
	
		INTEGER,INTENT(IN)::ntheta
		REAL,INTENT(IN)::theta,thetaArray(1:ntheta)
		INTEGER,INTENT(OUT)::iAngLow, iAngHi
	
		INTEGER:: iAng
	

		iAngLow = 1
		iAngHi = ntheta

		DO
			IF((iAngHi - iAngLow) == 1)EXIT
			iAng = (iAngHi + iAngLow)/2
			IF(thetaArray(iAng) < theta)THEN
				iAngLow = iAng
			ELSE
				iAngHi = iAng
			ENDIF
		ENDDO
	
		RETURN
	
		END SUBROUTINE BisectionSearch

	END SUBROUTINE GetReflForGivenWindSpeedNew
	!-----------------------------------------------------------------------------------------------
	
		
	!-------------------------- SUBROUTINE InterpolateEmissivity -----------------------------------
	!This program interpolates(linear) the emissivity array for a given solar/oview angle and returns
	!the corresponding values for particular tau, lambda and re (see the inputs and outputs)
	!INPUTS:
	!	solarOrViewAng		:	input angle in degrees (scalar)
	!	solarAngleMuArray	:	1-D array of solar angles for which emissivity vals are precomputed
	!	inFluxArray			:	4-D array of emissivity vals: 1st DIM = num of angles, 
	!							2nd DIM = ntaus, 3rd DIM = num of wavelengths, 4th DIM = num of REs
	!OUTPUTS:
	!	outFluxArray		: 3-D array ofinterpolated cloud or surface EM values at the input angle
	!						  1st dim = ntaus, 2nd dim = num of wavelengths, 3rdd dim = num of rE's
	!	ierror			:	integer value
	!						ierror 	= 0  	success, 
	!								= 3		invalid data
	!-----------------------------------------------------------------------------------------------
	! 						
	SUBROUTINE InterpolateEmissivity(solarOrViewAng, solarAngMuArray, inFluxArray, & 
																	outFluxArray, ierror)
	IMPLICIT NONE
	!Arguments
	REAL,INTENT(IN):: solarOrViewAng, solarAngMuArray(:),inFluxArray(:,:,:,:)
	REAL,INTENT(OUT)::outFluxArray(:,:,:)
	INTEGER,INTENT(OUT)::ierror

	!local variables
	INTEGER:: iAng, iAngLow, iAngHi
	REAL*8::dtheta,UMU
	CHARACTER(100)::tempstr

	!start esecution
	ierror = 0
	
	UMU  = COS(solarOrViewAng * RAD_PER_DEG)
	iAngLow = 1
	iAngHi = size(solarAngMuArray)
	IF(UMU < solarAngMuArray(iAngLow))THEN
		iAngHi = iAngLow+1 !no need to do the search; do extrapolation
		
	ELSE
		DO									!do a bisection search
			IF((iAngHi - iAngLow) == 1)EXIT
			iAng = (iAngHi + iAngLow)/2
			IF(solarAngMuArray(iAng) < UMU)THEN
			iAngLow = iAng
			ELSE
				iAngHi = iAng
			ENDIF
		ENDDO
	ENDIF


	dtheta = solarAngMuArray(iAngHi) - solarAngMuArray(iAngLow)
	IF(dtheta ==0)THEN
		ierror = 3    !invalid data read
		RETURN
	ENDIF


	outFluxArray(:,:,:) = ( inFluxArray(:,:,:,iAngLow)  * (solarAngMuArray(iAngHi)- UMU) + &
                    inFluxArray(:,:,:,iAngHi)  * (UMU - solarAngMuArray(iAngLow) ))/dtheta 
    RETURN

	RETURN

	END SUBROUTINE InterpolateEmissivity
	!-----------------------------------------------------------------------------------------------


	!-----------------------------SUBROUTINE TriLinearInterpolationRefl-----------------------------
	!perform 3-D linear interpolation on multiple scattering part of the reflectance. Instead of 
	!indexing, a search is done to find the the indices of the cube that enclose the interpolation
	!point.
	!INPUTS	:	
	!	solarAng		:	solar zenith angle in degrees (scalar)
	!	viewAng 		:	sensor zenith angle in degrees (scalar)
	!	azmAng			:	rel azm ang in degrees (scalar)]
	!	solarMuArray	:	array of solar zenith angles (from the LUT) 1-D array
	!	viewMuArray		:	array of sensor zenith angles (from the LUT) 1D array
	!	relAzmArray		:	array of rel azimuth angles (from the LUT) 1-D array
	!	multiScatReflLUT:	multi scat part of the reflectance (from LUT) 6D array
	!						DIMS[1:nview, 1:nsolar,1:nazm, 1:ntau, 1:nlambda, 1:nre]
	!OUTPUTS :
	!	multiScatRefl	:	interpolated MS reflectance for the input geometry 3D array
	!						DIMS[1:ntau, 1:nlambda, 1:nre]
	!	ierror			:	integer value
	!						ierror 	= 0  	success, 
	!								= 1 	I/O error
	!								= 2 	memory allocation error
	!								= 3		invalid data	
	!ROUTINES USED :
	!	BisectionSearch - internal procedure to this subroutine
	!-----------------------------------------------------------------------------------------------	

	SUBROUTINE TriLinearInterpolationRefl(solarAng,viewAng,azmAng,lutin, multiScatRefl,ierror)
    IMPLICIT NONE

	REAL,INTENT(IN):: solarAng, viewAng, azmAng
    TYPE(lutRefl),INTENT(IN)::lutin

	REAL,INTENT(OUT)::multiScatRefl(:,:,:)
	INTEGER,INTENT(OUT)::ierror


	REAL*8::prod11,prod12,prod21,prod22,vol1,vol2,vol3,vol4,vol5,vol6,vol7,vol8,volume
	REAL*8:: PI, torad
	REAL*4::solarmu,viewmu
	REAL*8::dmu0,dmu,dmu0_1,dmu0_2,dmu_1,dmu_2,dazm,dazm_1,dazm_2
	INTEGER:: ishi,islow, ivhi,ivlow, iazhi,iazlow,n1,n2,n3,allocateStatus
    REAL,ALLOCATABLE::solarMuArray(:),viewMuArray(:),relAzmArray(:)
    
	
	
	ierror = 0   !no error
	solarmu  = COS(solarAng * RAD_PER_DEG)
	viewmu   = COS(viewAng  * RAD_PER_DEG)
	

	n1 = size(lutin%solzen)
	n2 = size(lutin%senzen)
	n3 = size(lutin%relazm)
!	print*,n1,n2,n3
	
	allocate(solarMuArray(1:n1),viewMuArray(1:n2),relAzmArray(1:n3),STAT = allocateStatus)
				
	IF(allocateStatus /=0) THEN
		ierror = 2			!memory allocation error
		write(*,*)'Memory allocation error in subroutine TriLinearInterpolationRefl'
		stop
		ENDIF			 	
	
	
    ishi = lutin%idsol(2);islow = lutin%idsol(1)
 !   print*,ishi,islow
    ivhi = lutin%idzen(2);ivlow = lutin%idzen(1)
 !   print*,ivhi,ivlow
    iazhi = lutin%idazm(2);iazlow = lutin%idazm(1)
!    print*,iazlow,iazhi

    
    solarMuArray(:) = lutin%solzen(:)
    viewMuArray(:)  = lutin%senzen(:)
    relAzmArray     = lutin%relazm(:)
    

	dmu = solarMuArray(ishi) - solarMuArray(islow)
	dmu0  = viewMuArray(ivhi)  - viewMuArray(ivlow)
	dazm = relAzmArray(iazhi) - relAzmArray(iazlow)
	
	dmu_1 = solarMu - solarMuArray(islow) ; dmu_2 = solarMuArray(ishi) - solarMu
	dmu0_1  = viewMu  - viewMuArray(ivlow)  ; dmu0_2  = viewMuArray(ivhi)  - viewMu
	dazm_1 = azmAng  -  relAzmArray(iazlow); dazm_2 = relAzmArray(iazhi) - azmAng
		
	volume = dmu0 * dmu * dazm
	IF(volume == 0.0)THEN
		ierror = 3		!invalid data 
		RETURN
	ENDIF
	!print*,ivhi,ishi,iazlow
	!print*,dmu0_1,dmu0_2,dmu_1,dmu_2,dazm_1,dazm_2

	prod22 = dmu_2 * dmu0_2
	prod12 = dmu_1 * dmu0_2
	prod11 = dmu_1 * dmu0_1
	prod21 = dmu_2 * dmu0_1

	vol8 =  abs(prod22 * dazm_2)
	vol7 =  abs(prod22 * dazm_1)
	vol6 =  abs(prod12 * dazm_1)
	vol5 =  abs(prod12 * dazm_2)

	vol1 =  abs(prod11 * dazm_1)
	vol2 =  abs(prod21 * dazm_1)
	vol3 =  abs(prod21 * dazm_2)
	vol4 =  abs(prod11 * dazm_2)
	!volume = vol1 + vol2 + vol3 + vol4 + vol5 + vol6 + vol7 + vol8
!	print*,'comes here'
!	print*,ivlow,islow,iazlow
!	print*,size(multiScatRefl,1),size(multiScatReflLUT,4)
	multiScatRefl(:,:,:) = ( lutin%msr(:,:,:,1,1,1) * vol8  + &
							 lutin%msr(:,:,:,1,1,2)  * vol3  + &
                       		 lutin%msr(:,:,:,2,1,2)   * vol2  + &
                       		 lutin%msr(:,:,:,2,1,1)  * vol7  + &
					   		 lutin%msr(:,:,:,1,2,1)  * vol5  + &
					   		 lutin%msr(:,:,:,1,2,2)   * vol4  + &
                       		 lutin%msr(:,:,:,2,2,2)    * vol1  + &
                       		 lutin%msr(:,:,:,2,2,1)   * vol6 )/volume
    IF(allocated(solarMuArray))deallocate(solarMuArray)
    IF(allocated(viewMuArray))deallocate(viewMuArray)
    IF(allocated(relAzmArray))deallocate(relAzmArray)

    
    
 
	RETURN

	
	END SUBROUTINE TriLinearInterpolationRefl
	!-----------------------------------------------------------------------------------------------
	
	SUBROUTINE BisectionSearch(theta,thetaArray,ntheta,iAngLow, iAngHi)
	
		INTEGER,INTENT(IN)::ntheta
		REAL,INTENT(IN)::theta,thetaArray(1:ntheta)
		INTEGER,INTENT(OUT)::iAngLow, iAngHi
	
		INTEGER:: iAng
	

		iAngLow = 1
		iAngHi = ntheta

		DO
			IF((iAngHi - iAngLow) == 1)EXIT
			iAng = (iAngHi + iAngLow)/2
			IF(thetaArray(iAng) < theta)THEN
				iAngLow = iAng
			ELSE
				iAngHi = iAng
			ENDIF
		ENDDO
	
		RETURN
	
	END SUBROUTINE BisectionSearch

	
	
	!---------------------------SUBROUTINE WriteComments--------------------------------------------
	!write comments to the terminal screen and to a file opened with the given unit
	!INPUT	:
	!	comment	:	comment that needs to be written (string)
	!	unit	:	file unit (integer)
	SUBROUTINE WriteComments(comment, unit)

	CHARACTER(len = *), INTENT(IN)::comment
	INTEGER, INTENT(IN)::unit

	LOGICAL::isopened = .TRUE.

	WRITE(*,'(1X, A)')comment
	INQUIRE(unit, opened = isopened)
	IF(isopened)WRITE(unit,'(1X, A)')comment
	END SUBROUTINE WriteComments
	!-----------------------------------------------------------------------------------------------

	!---------------------------SUBROUTINE SetLogFileUnit-------------------------------------------
	SUBROUTINE SetLogFileUnit
	!subroutine to set the comment file unit
	IMPLICIT NONE
	INTEGER::unit, i
	LOGICAL::isopened

	unit = 1
	i = 0
	isopened = .FALSE.
	loop:DO
 		unit = unit + 1
		INQUIRE(unit, opened = isopened)
		IF(.not. isopened)then
		lu_PGM_status = unit	
		EXIT	
		ENDIF
	ENDDO loop
   	END SUBROUTINE SetLogFileUnit	
	!-----------------------------------------------------------------------------------------------
   	
   	
   	!-----------------------------SUBROUTINE OpenLogFile--------------------------------------------
	!Function to open the log/comment/error file
	!INPUTS:
	!	fName	:	name of the file
	!
	!-----------------------------------------------------------------------------------------------
	SUBROUTINE OpenLogFile(fName)

	IMPLICIT NONE
	CHARACTER(len = *), INTENT(IN):: fName
	INTEGER :: istat
   	
   	CALL SetLogFileUnit
   	OPEN(lu_PGM_status, & 
	     FILE = TRIM(fName), STATUS ='REPLACE', IOSTAT = istat, ACTION = 'WRITE')
	IF(istat /=0)WRITE(*, '(1X, A)')'Cannot open the program status file'
	RETURN

	END SUBROUTINE OpenLogFile
	!-----------------------------------------------------------------------------------------------
	
	!-----------------------------SUBROUTINE CloseLogFile-------------------------------------------
	!Function to open the log/comment/error file
	!INPUTS:
	!	fName	:	name of the file
	!
	!-----------------------------------------------------------------------------------------------
	SUBROUTINE CloseLogFile

	IMPLICIT NONE
	LOGICAL::isopened

   	
  	INQUIRE(lu_PGM_status, opened = isopened)
  	IF(isopened) CLOSE(lu_PGM_status)

	END SUBROUTINE CloseLogFile
	!-----------------------------------------------------------------------------------------------

    	!---------------------------getdateAndTime------------------------------------------
		!subroutine to get the current date and time as strings.
		!eg: date = SEP 9 1993, TIME = 12:59:46
		!INPUTS:
		!	NONE
		!OUTPUTS:
		!	date : string giving the current date
		!	time : string giving the time
		!ROUTINES CALLED:
		!fortran intrinsic routine date_and_time
		!-----------------------------------------------------------------------------------

		SUBROUTINE getdateAndTime(whatdate, whattime,dateTimeString)
		IMPLICIT NONE

		CHARACTER(11)::whatdate
		CHARACTER(8):: whattime
		CHARACTER(13)::dateTimeString

		CHARACTER(8)::date1
		CHARACTER(10)::time1
		CHARACTER(5)::zone
		CHARACTER(3)::dayoftheyear
		INTEGER::values(8),result,values1(9)
		INTEGER::Time

		CALL date_and_time(date1,time1,zone, values)
			     result = TIME()
	     
	     CALL GMTIME(result,values1)
	     

		 write( dayoftheyear,'(i3.3)')values1(8)+1
		 dateTimeString = date1(1:4)//dayoftheyear//time1(1:2)//time1(3:4)//time1(5:6)

		SELECT CASE(values(2))
			CASE(1)
				whatdate(1:3) = 'JAN'
			CASE(2)
				whatdate(1:3) = 'FEB'
			CASE(3)
				whatdate(1:3) = 'MAR'
			CASE(4)
				whatdate(1:3) = 'APR'
			CASE(5)
				whatdate(1:3) = 'MAY'
			CASE(6)
				whatdate(1:3) = 'JUN'
			CASE(7)
				whatdate(1:3) = 'JUL'
			CASE(8)
				whatdate(1:3) = 'AUG'
			CASE(9)
				whatdate(1:3) = 'SEP'
			CASE(10)
				whatdate(1:3) = 'OCT'
			CASE(11)
				whatdate(1:3) = 'NOV'
			CASE(12)
				whatdate(1:3) = 'DEC'
		END SELECT

		whatdate(4:11) = ' ' // date1(7:8) // ' ' // date1(1:4)
		whattime = time1(1:2)//':'//time1(3:4)//':'//time1(5:6)
		RETURN
		END SUBROUTINE getDateAndTime
		
		
   	!-----------------------------SUBROUTINE createHDFFile--------------------------------------------
	!Subroutine to create the output file in HDF4
	!INPUTS:
	!	fName	:	name of the output file file
	!
     !OUTPUTS:
     !			NONE yet
     !ROUTINES CALLED:
     !			getDateAndTime :  contains in the program itself
     !MODULES USED:
     !			HDF.f90, DFFUNC.f90
     !
     !----------------------------------------------------------------------------------------------
  
      
     SUBROUTINE createHDFFile(outfile,reflWater, reflIce,angs, rew,rei,tau, wvl, SURFALB, WSPEED)
     IMPLICIT NONE
     INCLUDE "hdf.f90"
    INCLUDE "dffunc.f90"

     
     CHARACTER(len=*),INTENT(IN)::outfile
     REAL, INTENT(IN)::angs(3), reflWater(:,:,:), reflIce(:,:,:),rew(:),rei(:),tau(:),wvl(:)
     REAL,INTENT(IN),OPTIONAL::SURFALB, WSPEED
     
     REAL::solar, view, azm

     INTEGER*4:: sd_id, sds_id, rank, dim_id,hdf_id,dimArray(7),start(7),edges(7),stride(7)
     integer*4:: DIM0,DIM1,DIM2,DIM3,DIM4,DIM5
     integer:: dims(2) !, start(2), edges(2), stride(2)
     integer:: i, j, k, retstat
     CHARACTER(200) :: dimensionName,SDSName,comment,longName

     CHARACTER(11)  :: whatdate
     CHARACTER(8)   :: whattime
     CHARACTER(13)::dateTimeString
          REAL*8::scaleFactor


	     CALL getdateAndTime(whatdate, whattime,dateTimeString)
	 


     hdf_id = sfstart(trim(outfile), DFACC_CREATE) 
     IF (hdf_id == Fail) THEN
     	WRITE(*,*)"Error in creating the file ", trim(outfile)
       	STOP
     ENDIF
 
       solar = angs(1)
       view=angs(2)
       azm=angs(3)

    !print*,whatdate,'  ',whattime
    if(present(SURFALB))THEN
        retstat = sfscatt(hdf_id, "TITLE", DFNT_CHAR, len(trim('TOA REFLECTANCE OVER LAND')), trim('TOA REFLECTANCE OVER LAND'))
        retstat = sfsnatt(hdf_id, "SURFACE ALBEDO", DFNT_FLOAT, 1, SURFALB) 
     endif
    if(present(WSPEED))THEN
        retstat = sfscatt(hdf_id, "TITLE", DFNT_CHAR, len(trim('TOA REFLECTANCE OVER OCEAN')), trim('TOA REFLECTANCE OVER OCEAN'))
        retstat = sfsnatt(hdf_id, "WIND_SPEED", DFNT_FLOAT, 1, WSPEED) 
        retstat = sfscatt(hdf_id, "WIND_SPEED_UNITS", DFNT_CHAR, len(trim('m/s')), trim('m/s'))
        
     endif
     
     retstat = sfsnatt(hdf_id, "SOLARANG", DFNT_FLOAT, 1, solar)
     retstat = sfsnatt(hdf_id, "VIEWANG", DFNT_FLOAT, 1, view)
     retstat = sfsnatt(hdf_id, "AZMANG", DFNT_FLOAT, 1, azm)
     retstat = sfsnatt(hdf_id, "CREATION_DATE", DFNT_CHAR, len(trim(whatdate//' '//whattime)), trim(whatdate//' '//whattime))
          
        
    
     
	!
  	!--------------Create the SDS, Water Reflectance-------------------------------------	
	rank  = 3
	start(:)=0
	stride(:)=1
	DIM2 = size(reflWater,3)
	DIM0 = size(reflWater,1)
	DIM1 = size(reflWater,2)
	dimArray(1:rank) = (/DIM0,DIM1,DIM2/)
	
	edges(1:rank)=dimArray(1:rank)
	

	SDSName = 'reflectanceWater'
	sds_id = SFcreate(hdf_id, trim(SDSName), DFNT_FLOAT, rank, dimArray(1:rank))
	retstat = sfwdata(sds_id, start(1:rank), stride(1:rank), edges(1:rank), reflWater)
	
	 dim_id = sfdimid(sds_id,1)
    dimensionName = ' NumOfWvl'
    retstat = sfsdmname(dim_id, trim(dimensionName))
  	
	dim_id = sfdimid(sds_id,0)
    dimensionName = 'NumOfCOT'
    retstat = sfsdmname(dim_id, trim(dimensionName))
	
	
	dim_id = sfdimid(sds_id,2)
    dimensionName = 'NumOfRE_Water'
    retstat = sfsdmname(dim_id, trim(dimensionName))
    
    	scaleFactor = 1.d0
        retstat = sfsnatt(sds_id, 'Scale_Factor', DFNT_DOUBLE, 1, scaleFactor)

    
     longname = 'Lambertian Water C6 Reflectance'
        retstat = sfscatt(sds_id, 'Discription', DFNT_CHAR, len(trim(longName)), trim(longName))

  
    
	retstat = sfendacc(sds_id)
	
	
	  	
  	!------Create the SDS Effective Droplet radiii--------------------------------------
	rank =1
	start(1)=0
	edges(1)=size(rew)
	stride(1)=1
	dimArray(1) = size(rew)
	
    SDSName = 'ParticleRadiusWater' !'EffectiveParticleRadii'
    sds_id = SFcreate(hdf_id, trim(SDSName), DFNT_FLOAT, rank, dimArray(1:rank))
    
    retstat = sfwdata(sds_id, start(1), stride(1), edges(1), rew)
    
    !Units
    	scaleFactor = 1.d0
    retstat = sfsnatt(sds_id, 'Scale_Factor', DFNT_DOUBLE, 1, scaleFactor)

    longname = 'Effective Radii for water'
        retstat = sfscatt(sds_id, 'Discription', DFNT_CHAR, len(trim(longName)), trim(longName))
    retstat = sfscatt(sds_id, 'Units', DFNT_CHAR, 8, 'microns')
       
  	retstat = sfendacc(sds_id)
  
  	!--------------Create the SDS, Ice Reflectance-------------------------------------	
	rank  = 3
	start(:)=0
	stride(:)=1
	DIM2 = size(reflIce,3)
	DIM0 = size(reflIce,1)
	DIM1 = size(reflIce,2)
	dimArray(1:rank) = (/DIM0,DIM1,DIM2/)
	
	edges(1:rank)=dimArray(1:rank)
	

	SDSName = 'reflectanceIce'
	sds_id = SFcreate(hdf_id, trim(SDSName), DFNT_FLOAT, rank, dimArray(1:rank))
	retstat = sfwdata(sds_id, start(1:rank), stride(1:rank), edges(1:rank), reflIce)
	

	 dim_id = sfdimid(sds_id,1)
    dimensionName = ' NumOfWvl'
    retstat = sfsdmname(dim_id, trim(dimensionName))
  	
	dim_id = sfdimid(sds_id,0)
    dimensionName = 'NumOfCOT'
    retstat = sfsdmname(dim_id, trim(dimensionName))
	
	
	dim_id = sfdimid(sds_id,2)
    dimensionName = 'NumOfRE_Ice'
    retstat = sfsdmname(dim_id, trim(dimensionName))
    
    	scaleFactor = 1.d0
    retstat = sfsnatt(sds_id, 'Scale_Factor', DFNT_DOUBLE, 1, scaleFactor)

    
      longname = 'Lambertian Ice C6 Reflectance'
        retstat = sfscatt(sds_id, 'Discription', DFNT_CHAR, len(trim(longName)), trim(longName))
  
    
	retstat = sfendacc(sds_id)
	
	
	  	
  	!------Create the SDS Effective Droplet radiii Ice--------------------------------------
	rank =1
	start(1)=0
	edges(1)=size(rei)
	stride(1)=1
	dimArray(1) = size(rei)
	
    SDSName = 'ParticleRadiusIce' !'EffectiveParticleRadii'
    sds_id = SFcreate(hdf_id, trim(SDSName), DFNT_FLOAT, rank, dimArray(1:rank))
    
    retstat = sfwdata(sds_id, start(1), stride(1), edges(1), rei)
    
    !Units
    	scaleFactor = 1.d0
    retstat = sfsnatt(sds_id, 'Scale_Factor', DFNT_DOUBLE, 1, scaleFactor)

    longname = 'Effective Radii for ice'
        retstat = sfscatt(sds_id, 'Discription', DFNT_CHAR, len(trim(longName)), trim(longName))
    retstat = sfscatt(sds_id, 'Units', DFNT_CHAR, 8, 'microns')
       
  	retstat = sfendacc(sds_id)

	
	
	!------Create the SDS Wavelengths for Reflectance-----------------------------------
	rank =1
	start(1)=0
	edges(1)=size(wvl)
	stride(1)=1
	dimArray(1) = size(wvl)
	
 !   SDSName = 'WaveLengthsReflectance'
 	SDSName = 'Wavelengths'

    sds_id = SFcreate(hdf_id, trim(SDSName), DFNT_FLOAT, rank, dimArray(1:rank))
    
    retstat = sfwdata(sds_id, start(1), stride(1), edges(1), wvl)
   
    !Units
    	scaleFactor = 1.d0
    retstat = sfsnatt(sds_id, 'Scale_Factor', DFNT_DOUBLE, 1, scaleFactor)

        longName = 'Wavelengths LUT'
    retstat = sfscatt(sds_id, 'Discription', DFNT_CHAR8, len(trim(longName)),trim(longName))

    retstat = sfscatt(sds_id, 'Units', DFNT_CHAR, 8, 'microns')
   
    retstat = sfendacc(sds_id)
	

	!------Create the SDS Optical Thickness---------------------------------------------
	rank =1
	start(1)=0
	edges(1)=size(tau)
	stride(1)=1
	dimArray(1) = size(tau)
	
    SDSName = 'OpticalThickness'
    sds_id = SFcreate(hdf_id, trim(SDSName), DFNT_FLOAT, rank, dimArray(1:rank))
    
    retstat = sfwdata(sds_id, start(1), stride(1), edges(1), tau)
   
    !Units
    	scaleFactor = 1.d0
    retstat = sfsnatt(sds_id, 'Scale_Factor', DFNT_DOUBLE, 1, scaleFactor)

            longName = 'COT LUT'
    retstat = sfscatt(sds_id, 'Discription', DFNT_CHAR8, len(trim(longName)),trim(longName))

    retstat = sfscatt(sds_id, 'Units', DFNT_CHAR, 4, 'none')
    
  	retstat = sfendacc(sds_id)
  	
  	
	!------------------------Close the HDF File-----------------------------------------
  	retstat = sfend(hdf_id)
  	END SUBROUTINE createHDFFile


   
    

END MODULE interpolate_C6_LUT_m



