
; NAME :    
;           getReflForGivenSurfAlbedo_flip
; PURPOSE :
;           Calculate reflectance over land surface, given the satellite geometry, surface albedo
;           from MS C6 LUTs
; CALLING :
;           liquid_refl = getReflForGivenSurfAlbedo_flip(pf,lut,ssc,sol,view,azm,surfAlb,'LIQUID')
;           ice_refl    = getReflForGivenSurfAlbedo_flip(pf,lut,ssc,sol,view,azm,surfAlb,'ICE')
;                        sol, view, azm can be scalars or vectors of the same length
;                        surfAlb can be a scalar or a vector
; INPUTS  :
;           pf  = Structure containing phasefunction data
;           lut = Structure containing MS reflectances for the LAND LUT 
;           ssc = Structure with Single scattering properties and LUT RE & tau values
;           sol    = Solar Angle in degrees
;           view   = View Angle in degrees
;           azm    = Azimuth Angle in degrees
;           surfAlb = Surface Albedo Value
;           PHASE     = Liquid or Ice phase
; OUTPUT  :
;           Array containing reflectance over land
;           1st DIM = DIMS of input surface albedo vector
;           2nd DIM = DIMS of sol, view, azm angle vector (similar dims for three angles)
; 
; EXTERNAL CALLS:     
;           calcSingScatPartOfReflectancenew  (calculate SS part) 
;           interpolateMSRefl_flip (interpolate MS reflectance)
;           interpolateFluxes_flip (interpolate fluxes, sp albedo etc)
;              
; HISTORY:
; Written: Nandana Amarasinghe, SSAI @ NASA/GSFC, 2017.
;



FUNCTION getReflForGivenSurfAlbedo_flip,pfStruct,lutStruct,sscStruct,solarAng,viewAng,azmAng,surfAlb,phase

  calcSingScatPartOfReflectancenew,pfStruct,sscStruct,solarAng,viewAng,azmAng,ssRefl, PHASE=phase
  interpolateMSRefl_flip, lutstruct,interpMSR
  ;help,ssRefl
  ;help,interpMSR
  reflAsurf = ssRefl + transpose(interpMSR)

  reflFlux    = lutStruct.RFLUX
  transFlux   = lutStruct.TFLUX
  spAlbedo    = lutStruct.SPALB
  solarMuRFTF = lutStruct.FLUXSOL
  
  nsol = n_elements(solarAng)
   nsalb = n_elements(surfAlb)
  

  interpolateFluxes_flip, lutstruct.idsol, transFlux, outTFluxArrayMu0
  interpolateFluxes_flip, lutstruct.idsenf, reflFlux, outTFluxArrayMU
 
   DIMS = size(reflAsurf, /DIMENSIONS)
  temprefl = fltarr([nsalb,DIMS])
  
  
  
 
  for k = 0,nsalb-1 do begin
   temprefl[k,*,*,*,*] = reflAsurf + $
     reform(reform((surfAlb[k] * transpose(outTFluxArrayMu0) * transpose(outTFluxArrayMU)),nsol,DIMS[1]*DIMS[2]*DIMS[3])#$
     diag_matrix(reform(1.0 / ( 1.0 - surfAlb[k] * spAlbedo),DIMS[1]*DIMS[2]*DIMS[3])),nsol,DIMS[1],DIMS[2],DIMS[3])
  end
;  
  RETURN, REFORM(temprefl)
 end